'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);

var _reactMeasure = require('react-measure');

var _theme = require('./theme');

var _theme2 = _interopRequireDefault(_theme);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var GENE_SLOT_HEIGHT = 30;
var GENE_TRANSCRIPT_HEIGHT = 7;
var GENE_TRANSCRIPT_OFFSET = 2;
var GENE_BACKDROP_PADDING = 2;
var GENE_LABEL_OFFSET = GENE_TRANSCRIPT_OFFSET * 2 + GENE_TRANSCRIPT_HEIGHT;
var GENE_TRACK_PADDING = 5;
var STUDY_SLOT_HEIGHT = 30;
var STUDY_TEXT_MAX_WIDTH = 200;
var VARIANT_TRACK_HEIGHT = 7;
var CONNECTOR_TRACK_HEIGHT = 80;
var CHAR_WIDTH = 12; // TODO: base on font-size
var HEIGHT_DEFAULT = 400;

var Gecko = function (_React$Component) {
  _inherits(Gecko, _React$Component);

  function Gecko(props) {
    _classCallCheck(this, Gecko);

    var _this = _possibleConstructorReturn(this, (Gecko.__proto__ || Object.getPrototypeOf(Gecko)).call(this, props));

    _this.svgRef = _react2.default.createRef();
    return _this;
  }

  _createClass(Gecko, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this._render();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this._render();
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var measureRef = this.props.measureRef;

      var _dimensions2 = this._dimensions(),
          width = _dimensions2.width,
          height = _dimensions2.height;

      return _react2.default.createElement(
        'div',
        { ref: measureRef },
        _react2.default.createElement('svg', { width: width, height: height, ref: function ref(node) {
            return _this2.svgRef = node;
          } })
      );
    }
  }, {
    key: '_width',
    value: function _width() {
      var contentRect = this.props.contentRect;
      var width = contentRect.bounds.width;

      return width;
    }
  }, {
    key: '_height',
    value: function _height() {
      var _props = this.props,
          data = _props.data,
          start = _props.start,
          end = _props.end;

      var width = this._width();
      if (!width || !data || !data.tagVariants || !data.genes || !data.geneTagVariants) {
        return HEIGHT_DEFAULT;
      }
      var scalePosition = d3.scaleLinear().domain([start, end]).range([0, width]);

      var _geneSlots2 = this._geneSlots(scalePosition),
          geneSlots = _geneSlots2.slots;

      var studySlotCount = this._studySlotCount(width);
      var trackConfig = this._trackConfig(geneSlots.length, studySlotCount);
      return trackConfig.height;
    }
  }, {
    key: '_dimensions',
    value: function _dimensions() {
      var width = this._width();
      var height = this._height();
      return { width: width, height: height };
    }
  }, {
    key: '_render',
    value: function _render() {
      var _props2 = this.props,
          data = _props2.data,
          start = _props2.start,
          end = _props2.end;

      var width = this._width();
      if (!width || !data || !data.tagVariants || !data.genes || !data.geneTagVariants) {
        return null;
      }

      var svg = d3.select(this.svgRef);
      var scalePosition = d3.scaleLinear().domain([start, end]).range([0, width]);

      var _geneSlots3 = this._geneSlots(scalePosition),
          geneSlots = _geneSlots3.slots,
          genesWithSlots = _geneSlots3.genesWithSlots;

      var studySlotCount = this._studySlotCount(width);
      var trackConfig = this._trackConfig(geneSlots.length, studySlotCount);

      var _studyScales2 = this._studyScales(width, trackConfig.studies.height),
          scaleStudyX = _studyScales2.scaleStudyX,
          scaleStudyY = _studyScales2.scaleStudyY;

      this._renderGenes(svg, scalePosition, trackConfig.genes, genesWithSlots);
      this._renderTagVariants(svg, scalePosition, trackConfig.tagVariants, data.tagVariants);
      this._renderIndexVariants(svg, scalePosition, trackConfig.indexVariants, data.indexVariants);
      this._renderGeneTagVariants(svg, scalePosition, trackConfig.geneTagVariants, data.geneTagVariants);
      this._renderStudies(svg, scaleStudyX, scaleStudyY, trackConfig.studies, data.studies);
      this._renderTagVariantIndexVariantStudies(svg, scalePosition, scaleStudyX, trackConfig.tagVariantIndexVariants, trackConfig.indexVariantStudies, data.tagVariantIndexVariantStudies);
    }
  }, {
    key: '_renderGenes',
    value: function _renderGenes(svg, scale, track, data) {
      // render in own group
      var g = svg.select('.track.track--genes');
      if (g.empty()) {
        g = svg.append('g').classed('track track--genes', true);
      }
      g.attr('transform', 'translate(0,' + track.top + ')');

      // join
      var genes = g.selectAll('g').data(data, function (d) {
        return d.id;
      });

      // -------------
      // --- GENES ---
      // -------------

      // --- ENTER ---
      var enter = genes.enter().append('g');

      // vertical
      enter.append('line').classed('vertical', true).attr('stroke', _theme2.default.connector.color);

      // backdrop
      enter.append('rect').classed('backdrop', true).attr('fill', 'white').attr('stroke', _theme2.default.line.color);

      // spit
      enter.append('line').classed('spit', true).attr('stroke', _theme2.default.line.color);

      // label
      enter.append('text').classed('label', true).attr('fill', _theme2.default.line.color).attr('alignment-baseline', 'hanging').attr('font-family', 'sans-serif').attr('font-size', 12);

      // --- MERGE ---
      var merge = enter.merge(genes).attr('transform', function (d) {
        return 'translate(0,' + d.slotIndex * GENE_SLOT_HEIGHT + ')';
      });

      // vertical
      merge.selectAll('line.vertical').attr('x1', function (d) {
        return scale(d.tss);
      }).attr('y1', GENE_TRANSCRIPT_OFFSET + GENE_TRANSCRIPT_HEIGHT / 2).attr('x2', function (d) {
        return scale(d.tss);
      }).attr('y2', function (d) {
        return track.height - d.slotIndex * GENE_SLOT_HEIGHT;
      });

      // backdrop
      merge.selectAll('rect.backdrop').attr('x', function (d) {
        return scale(d.start) - GENE_BACKDROP_PADDING;
      }).attr('y', GENE_TRANSCRIPT_OFFSET - GENE_BACKDROP_PADDING).attr('width', function (d) {
        var partnerText = d3.select(this.parentNode).select('text.label');
        var widthSpit = scale(d.end) - scale(d.start);
        var widthText = partnerText.node().getComputedTextLength();
        return Math.max(widthSpit, widthText) + GENE_BACKDROP_PADDING * 2;
      }).attr('height', GENE_TRANSCRIPT_HEIGHT + CHAR_WIDTH + GENE_BACKDROP_PADDING * 2);

      // spit
      merge.selectAll('line.spit').attr('x1', function (d) {
        return scale(d.start);
      }).attr('y1', GENE_TRANSCRIPT_OFFSET + GENE_TRANSCRIPT_HEIGHT / 2).attr('x2', function (d) {
        return scale(d.end);
      }).attr('y2', GENE_TRANSCRIPT_OFFSET + GENE_TRANSCRIPT_HEIGHT / 2);

      // label
      merge.selectAll('text.label').attr('x', function (d) {
        return scale(d.start);
      }).attr('y', GENE_LABEL_OFFSET).text(function (d) {
        return d.start === d.tss ? d.symbol + '>' : '<' + d.symbol;
      });

      // --- EXIT ---
      genes.exit().remove();

      // -------------
      // --- EXONS ---
      // -------------
      var exons = genes.selectAll('rect.exon').data(function (d) {
        return d.exons;
      });

      exons.enter().append('rect').attr('stroke', _theme2.default.line.color).attr('fill', 'white').classed('exon', true).merge(exons).attr('x', function (d) {
        return scale(d[0]);
      }).attr('y', GENE_TRANSCRIPT_OFFSET).attr('width', function (d) {
        return scale(d[1]) - scale(d[0]);
      }).attr('height', GENE_TRANSCRIPT_HEIGHT);

      exons.exit().remove();
    }
  }, {
    key: '_renderTagVariants',
    value: function _renderTagVariants(svg, scale, track, data) {
      // render in own group
      var g = svg.select('.track.track--tag-variants');
      if (g.empty()) {
        g = svg.append('g').classed('track track--tag-variants', true);
        g.append('rect');
      }
      g.attr('transform', 'translate(0,' + track.top + ')');

      // backdrop
      g.select('rect').attr('fill', _theme2.default.track.background).attr('x', 0).attr('y', 0).attr('width', scale.range()[1] - scale.range()[0]).attr('height', track.height);

      // join
      var tags = g.selectAll('line').data(data);

      tags.enter().append('line').attr('stroke', _theme2.default.line.color).attr('stroke-width', _theme2.default.line.thickness).merge(tags).attr('x1', function (d) {
        return scale(d.position);
      }).attr('y1', 0).attr('x2', function (d) {
        return scale(d.position);
      }).attr('y2', track.height);

      tags.exit().remove();
    }
  }, {
    key: '_renderIndexVariants',
    value: function _renderIndexVariants(svg, scale, track, data) {
      // render in own group
      var g = svg.select('.track.track--index-variants');
      if (g.empty()) {
        g = svg.append('g').classed('track track--index-variants', true);
        g.append('rect');
      }
      g.attr('transform', 'translate(0,' + track.top + ')');

      // backdrop
      g.select('rect').attr('fill', _theme2.default.track.background).attr('x', 0).attr('y', 0).attr('width', scale.range()[1] - scale.range()[0]).attr('height', track.height);

      // join
      var tags = g.selectAll('line').data(data);

      tags.enter().append('line').attr('stroke', _theme2.default.line.color).attr('stroke-width', _theme2.default.line.thickness).merge(tags).attr('x1', function (d) {
        return scale(d.position);
      }).attr('y1', 0).attr('x2', function (d) {
        return scale(d.position);
      }).attr('y2', track.height);

      tags.exit().remove();
    }
  }, {
    key: '_renderStudies',
    value: function _renderStudies(svg, scaleX, scaleY, track, data) {
      // render in own group
      var g = svg.select('.track.track--index-studies');
      if (g.empty()) {
        g = svg.append('g').classed('track track--index-studies', true);
      }
      g.attr('transform', 'translate(0,' + track.top + ')');

      // verticals
      var verticals = g.selectAll('line').data(data);

      verticals.enter().append('line').attr('stroke', _theme2.default.connector.color).merge(verticals).attr('x1', function (d) {
        return scaleX(d.studyId);
      }).attr('y1', 0).attr('x2', function (d) {
        return scaleX(d.studyId);
      }).attr('y2', function (d) {
        return scaleY(d.studyId);
      });

      verticals.exit().remove();

      // backdrops
      var backdrops = g.selectAll('rect').data(data);

      backdrops.enter().append('rect').attr('stroke', _theme2.default.line.color).attr('fill', 'white').merge(backdrops).attr('x', function (d) {
        return scaleX(d.studyId) - (CHAR_WIDTH * d.studyId.length + 2 * GENE_BACKDROP_PADDING) / 2;
      }).attr('y', function (d) {
        return scaleY(d.studyId);
      }).attr('width', function (d) {
        return CHAR_WIDTH * d.studyId.length + 2 * GENE_BACKDROP_PADDING;
      }).attr('height', CHAR_WIDTH + 2 * GENE_BACKDROP_PADDING);

      backdrops.exit().remove();

      // pointers
      var pointers = g.selectAll('circle').data(data);

      pointers.enter().append('circle').attr('stroke', _theme2.default.line.color).attr('fill', 'white').merge(pointers).attr('cx', function (d) {
        return scaleX(d.studyId);
      }).attr('cy', function (d) {
        return scaleY(d.studyId);
      }).attr('r', 2);

      pointers.exit().remove();

      // labels
      var texts = g.selectAll('text').data(data);

      texts.enter().append('text').attr('fill', _theme2.default.line.color).attr('text-anchor', 'middle').attr('font-family', 'sans-serif').attr('font-size', 12).merge(texts).attr('x', function (d) {
        return scaleX(d.studyId);
      }).attr('y', function (d) {
        return scaleY(d.studyId) + CHAR_WIDTH;
      }).text(function (d) {
        return d.studyId;
      });

      texts.exit().remove();
    }
  }, {
    key: '_renderGeneTagVariants',
    value: function _renderGeneTagVariants(svg, scale, track, data) {
      // render in own group
      var g = svg.select('.track.track--gene-tag-variants');
      if (g.empty()) {
        g = svg.append('g').classed('track track--gene-tag-variants', true);
      }
      g.attr('transform', 'translate(0,' + track.top + ')');

      // join
      var tags = g.selectAll('path').data(data);

      tags.enter().append('path').attr('stroke', _theme2.default.connector.color).attr('fill', 'none').merge(tags).attr('d', function (d) {
        var topX = scale(d.geneTss);
        var topY = 0;
        var bottomX = scale(d.tagVariantPosition);
        var bottomY = track.height;
        var controlY = (bottomY + topY) / 2;
        return 'M' + topX + ',' + topY + ' C' + topX + ',' + controlY + ', ' + bottomX + ',' + controlY + ' ' + bottomX + ',' + bottomY;
      });

      tags.exit().remove();
    }
  }, {
    key: '_renderTagVariantIndexVariantStudies',
    value: function _renderTagVariantIndexVariantStudies(svg, scaleGenome, scaleStudyX, tvIvTrack, ivSTrack, data) {
      // render in own group
      var g = svg.select('.track.track--tag-variant-index-variant-studies');
      if (g.empty()) {
        g = svg.append('g').classed('track track--tag-variant-index-variant-studies', true);
      }
      // g.attr('transform', `translate(0,${tvIvTrack.top})`);

      // join
      var tags = g.selectAll('path').data(data);

      tags.enter().append('path').attr('stroke', function (d) {
        return d.posteriorProbability ? _theme2.default.connector.finemappingColor : _theme2.default.connector.color;
      }).attr('fill', 'none').merge(tags).attr('d', function (d) {
        // tag variant - index variant
        var tvIvTopX = scaleGenome(d.tagVariantPosition);
        var tvIvTopY = tvIvTrack.top;
        var tvIvBottomX = scaleGenome(d.indexVariantPosition);
        var tvIvBottomY = tvIvTrack.bottom;
        var tvIvControlY = (tvIvBottomY + tvIvTopY) / 2;

        // index variant - study
        var ivSTopX = scaleGenome(d.indexVariantPosition);
        var ivSTopY = ivSTrack.top;
        var ivSBottomX = scaleStudyX(d.studyId);
        var ivSBottomY = ivSTrack.bottom;
        var ivSControlY = (ivSBottomY + ivSTopY) / 2;

        // render path
        return '\n          M' + tvIvTopX + ',' + tvIvTopY + ' C' + tvIvTopX + ',' + tvIvControlY + ', ' + tvIvBottomX + ',' + tvIvControlY + ' ' + tvIvBottomX + ',' + tvIvBottomY + '\n          M' + ivSTopX + ',' + ivSTopY + ' C' + ivSTopX + ',' + ivSControlY + ', ' + ivSBottomX + ',' + ivSControlY + ' ' + ivSBottomX + ',' + ivSBottomY + '\n        ';
      });

      tags.exit().remove();
    }
  }, {
    key: '_geneSlots',
    value: function _geneSlots(scale) {
      var genes = this.props.data.genes;

      var sortedGenes = genes.slice().sort(function (a, b) {
        return a.start - b.start;
      });

      var slotCount = 0;
      var slots = [];
      var genesWithSlots = [];
      sortedGenes.forEach(function (gene) {
        var suitableSlots = slots.filter(function (slot) {
          return scale(gene.start) > scale(slot.end) + gene.symbol.length * CHAR_WIDTH;
        });
        if (suitableSlots.length > 0) {
          // store in slots
          suitableSlots[0].genes.push(gene);
          suitableSlots[0].end = gene.end;

          // store in gene list with slot index
          genesWithSlots.push(_extends({}, gene, { slotIndex: suitableSlots[0].index }));
        } else {
          // store in slots
          var newSlot = { genes: [gene], end: gene.end, index: slotCount++ };
          slots.push(newSlot);

          // store in gene list with slot index
          genesWithSlots.push(_extends({}, gene, { slotIndex: newSlot.index }));
        }
      });

      return { slots: slots, genesWithSlots: genesWithSlots };
    }
  }, {
    key: '_studySlotCount',
    value: function _studySlotCount(width) {
      var studies = this.props.data.studies;

      var studiesPerSlot = Math.ceil(width / STUDY_TEXT_MAX_WIDTH);
      var slotCount = Math.ceil(studies.length / studiesPerSlot);
      return slotCount;
    }
  }, {
    key: '_studyScales',
    value: function _studyScales(width, trackHeight) {
      var studies = this.props.data.studies;

      var studiesPerSlot = Math.ceil(width / STUDY_TEXT_MAX_WIDTH);
      var slotCount = Math.ceil(studies.length / studiesPerSlot);
      var domain = studies.slice().sort(function (a, b) {
        return a.traitReported - b.traitReported;
      }).map(function (d) {
        return d.studyId;
      });
      var rangeX = [STUDY_TEXT_MAX_WIDTH / 2, width - STUDY_TEXT_MAX_WIDTH / 2];
      var rangeY = domain.map(function (d, i) {
        var s = GENE_TRACK_PADDING;
        var e = trackHeight - 2 * GENE_TRACK_PADDING;
        return s + (e - s) * (i % slotCount) / slotCount;
      });
      var scaleStudyX = d3.scalePoint().domain(domain).range(rangeX);
      var scaleStudyY = d3.scaleOrdinal().domain(domain).range(rangeY);

      return { scaleStudyX: scaleStudyX, scaleStudyY: scaleStudyY };
    }
  }, {
    key: '_trackConfig',
    value: function _trackConfig(geneSlotCount, studySlotCount) {
      var geneTrackHeight = geneSlotCount * GENE_SLOT_HEIGHT + 2 * GENE_TRACK_PADDING;
      var genes = {
        top: 0,
        bottom: geneTrackHeight,
        height: geneTrackHeight
      };
      var geneTagVariants = {
        top: genes.bottom,
        bottom: genes.bottom + CONNECTOR_TRACK_HEIGHT,
        height: CONNECTOR_TRACK_HEIGHT
      };
      var tagVariants = {
        top: geneTagVariants.bottom,
        bottom: geneTagVariants.bottom + VARIANT_TRACK_HEIGHT,
        height: VARIANT_TRACK_HEIGHT
      };
      var tagVariantIndexVariants = {
        top: tagVariants.bottom,
        bottom: tagVariants.bottom + CONNECTOR_TRACK_HEIGHT,
        height: CONNECTOR_TRACK_HEIGHT
      };
      var indexVariants = {
        top: tagVariantIndexVariants.bottom,
        bottom: tagVariantIndexVariants.bottom + VARIANT_TRACK_HEIGHT,
        height: VARIANT_TRACK_HEIGHT
      };
      var indexVariantStudies = {
        top: indexVariants.bottom,
        bottom: indexVariants.bottom + CONNECTOR_TRACK_HEIGHT,
        height: CONNECTOR_TRACK_HEIGHT
      };
      var studyTrackHeight = studySlotCount * STUDY_SLOT_HEIGHT + 2 * GENE_TRACK_PADDING;
      var studies = {
        top: indexVariantStudies.bottom,
        bottom: indexVariantStudies.bottom + studyTrackHeight,
        height: studyTrackHeight
      };
      return {
        genes: genes,
        geneTagVariants: geneTagVariants,
        tagVariants: tagVariants,
        tagVariantIndexVariants: tagVariantIndexVariants,
        indexVariants: indexVariants,
        indexVariantStudies: indexVariantStudies,
        studies: studies,
        height: studies.bottom
      };
    }
  }]);

  return Gecko;
}(_react2.default.Component);

exports.default = (0, _reactMeasure.withContentRect)('bounds')(Gecko);